@@ -49,6 +49,12 @@ typedef struct {
 	int depth;
 } win_env_t;
 
+typedef struct {
+	size_t size;
+	char *p;
+	char *buf;
+} win_bar_t;
+
 typedef struct {
 	Window xwin;
 	win_env_t env;
@@ -56,7 +62,6 @@ typedef struct {
 	unsigned long bgcol;
 	unsigned long fscol;
 	unsigned long selcol;
-	Pixmap pm;
 
 	int x;
 	int y;
@@ -66,10 +71,16 @@ typedef struct {
 
 	bool fullscreen;
 
+	struct {
+		int w;
+		int h;
+		Pixmap pm;
+	} buf;
+
 	struct {
 		unsigned int h;
-		char l[BAR_L_LEN];
-		char r[BAR_R_LEN];
+		win_bar_t l;
+		win_bar_t r;
 		unsigned long bgcol;
 		unsigned long fgcol;
 	} bar;
@@ -82,17 +93,13 @@ void win_open(win_t*);
 void win_close(win_t*);
 
 bool win_configure(win_t*, XConfigureEvent*);
-void win_expose(win_t*, XExposeEvent*);
 
 void win_toggle_fullscreen(win_t*);
 void win_toggle_bar(win_t*);
 
 void win_clear(win_t*);
 void win_draw(win_t*);
-void win_draw_rect(win_t*, Pixmap, int, int, int, int, bool, int,
-                   unsigned long);
-
-void win_update_bar(win_t*);
+void win_draw_rect(win_t*, int, int, int, int, bool, int, unsigned long);
 
 int win_textwidth(const char*, unsigned int, bool);
 
